
/*
 * CUDA PWM - Mathieu Giraud & Jean-Stéphane Varré, 2008
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>

int conf_N = 5 ;
int conf_print_plots = 0 ;
int conf_verbose = 0 ;
int conf_last_combine = 1 ;

#include "pwm.h"

#include "time_tools.c"
#include "cuda_tools.c"
#include "matrix.c"
#include "device_tools.c"
#include "word_tools.c"


// Score buckets 

#include "histograms.c"
#include "score_buckets.c"

// Scan

#include "scan.c"



void usage(char *progname)
{
	printf("Usage: %s [options] matrix.pfm [matrix2.pfm ...] \n", progname);

	printf("\n  -S    Scan (N matrices)\n");
	printf("        -T #  set the score threshold\n");
	printf("        -l #  random sequence length\n");
	printf("        -s #  sequence file\n");

	printf("\n  -B    BucketScoreDistribution %s\n",

#ifdef COMPARE
	       ": compare (EXPERIMENTAL) (2 matrices)"
#else
	       ": pvalue-to-score / score-to-pvalue (1 matrix)"
#endif
	       );


	printf("         -P #  set the Pvalue exponent (score-to-pvalue) (0 loops on all exponents)\n");
	printf("         -T #  set the score threshold (pvalue-to-score)\n");


	printf("\n         algorithm options\n");
	printf("         -N #  set the number of slices\n");
	printf("         -c    do not do the last combine (pvalue-to-score only)\n");
	printf("\n         verbose options\n");
	printf("         -p    print plots for the score distributions\n");
	printf("         -v    verbose\n");
	      
}


int main(int argc, char **argv)
{
	// print_sizeofs();
	
	extern int optind ;
	extern char *optarg ;
	char c ;
	int conf_Pvalue = 5 ;
	float conf_threshold = 0.0 ;

	int conf_scan = 0 ;
	int conf_buck = 0 ;
	int conf_random_length = 50 * 1024 * 1024 ;
	const char* conf_seq_file = NULL ; 

	while ((c = getopt(argc, argv, "P:T:pvcN:SBl:s:")) != EOF)
		switch(c)
		{
		case 'N':
			conf_N = atoi(optarg) ;
			break ;

		case 'l':
			conf_random_length = atoi(optarg) ;
			break ;

		case 'T':
			conf_threshold = atof(optarg) ;
			break ;

		case 'P':
		        conf_Pvalue = atoi(optarg) ;
			if (!conf_Pvalue)
		                conf_last_combine = 0 ;
			break ;

		case 'p':
			conf_print_plots = 1 ;
			break ;

		case 's':
			conf_seq_file = optarg ;
			break ;

		case 'v':
			conf_verbose = 1 ;
			break ;

		case 'c':
			conf_last_combine = 1 ;
			break ;


		case 'S':
			conf_scan = 1 ;
			break ;

		case 'B':
		        conf_buck = 1 ;
			break ;

		default:
			usage(argv[0]);
			return 1 ;
		}

	if (conf_scan + conf_buck == 0)
	  {
	    printf("One command (-S / -B) is mandatory.\n");
	    usage(argv[0]);
	    exit(1);
	  }

	
        printf("== prog: %s%s ", argv[0], PROG_FLAVOUR);
	printf(" -N %d -T %f -P %d \n", conf_N, conf_threshold, conf_Pvalue);
	 

	// read the matrices
	int nb_matrices = argc - optind ;

	if (!nb_matrices)
	{
		printf("!! No matrix given : using default matrix %s \n", DFT_MATRIX);
		nb_matrices = 1 ;		
	}

	float *matrices = (float *) malloc(nb_matrices * ALPH_SIZE * (MATRIX_MAX_LENGTH) * sizeof(float));
	int *lengths = (int *) malloc(nb_matrices * sizeof(int)) ;
	int offset = 0 ;

	int i = 0 ;
	while ((optind < argc) || (i == 0))
	{
		if (optind == argc)
			matrix_from_file(matrices, DFT_MATRIX, lengths);
		else
			matrix_from_file(matrices + offset, argv[optind++], lengths + i) ;
		
		offset += ALPH_SIZE * lengths[i] ;
		i++ ;

		if (conf_buck)
	  {
	    if (conf_N * MAX_LENGTH_PER_SLICE < lengths[i])
			{
				printf("Matrix too large : please use a larger N, or increase MAX_LENGTH_PER_SLICE.\n");
				exit (6);
			}

	  }
	}
	
	printf(" <--  %d matrices, total length %d \n", nb_matrices, offset / ALPH_SIZE);

	    
#ifdef COMPARE
	if (! (lengths[0] == lengths[1]))
	      {
		printf("Prototype : the two matrices must have the same length.\n");
		exit(6);
	      }
#endif

	// float *matrix = random_matrix(length) ;


	int seq_length ;
	unsigned char *seq ;

	if (conf_scan)
	  {

			if (conf_seq_file)
				seq = seq_from_file(conf_seq_file, &seq_length);
					
			else
			{
				// random seq
				seq_length = conf_random_length ;
				seq = random_seq(seq_length + MATRIX_MAX_LENGTH);
			}
	  }

	if (optind<argc) // non atteint, TODO
	{
		printf("Too many arguments.\n");
		usage(argv[0]);
		return 1 ;
	}


	// ----------------------- 

	time_start();  
	init_cuda();

	if (conf_scan)
	  scan(matrices, nb_matrices, lengths, seq, seq_length, conf_threshold);


	if (conf_buck)
	  {

	    pvalue2score_n(matrices, 
#ifdef COMPARE
			   matrices + lengths[0],
#endif
				 lengths[0], conf_N, conf_Pvalue, conf_threshold);  
	  }


	printf("==\n");
	time_stop();	

}
